// $Id: CommRenderer.java,v 1.3 2001/05/28 18:24:28 pcharles Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.client;

import java.util.Iterator;
import java.util.HashSet;
import java.awt.Color;
import java.awt.Canvas;
import java.awt.Graphics;
import net.sourceforge.jpcap.net.Packet;
import net.sourceforge.jpcap.net.TCPPacket;
import net.sourceforge.jpcap.net.UDPPacket;
import net.sourceforge.jpcap.net.ICMPPacket;


/**
 * A rendering of communication between two hosts.
 *
 * @author Patrick Charles and Jonas Lehmann
 * @version $Revision: 1.3 $
 * @lastModifiedBy $Author: pcharles $
 * @lastModifiedAt $Date: 2001/05/28 18:24:28 $
 */
public class CommRenderer
{
  /**
   * Create a new communication renderer.
   * @param canvas the canvas where this communication is being drawn.
   */
  public CommRenderer(Canvas canvas, 
                      Packet packet, 
                      HostRenderer hostA, HostRenderer hostB, 
                      String description) {
    this.canvas = canvas;
    this.packet = packet;
    this.hostA = hostA;
    this.hostB = hostB;
    this.description = description;

    // force paint of self when first created
    paint();
  }

  void paint() {
    draw(false);
  }

  void erase() {
    draw (true);
  }

  /**
   * Paint this communication on the canvas.
   */
  void draw(boolean erase) {
    if(gc == null)
      gc = canvas.getGraphics();

    int ax = hostA.getRenderedX(); int ay = hostA.getRenderedY();
    int bx = hostB.getRenderedX(); int by = hostB.getRenderedY();

    int offsetx = 0;
    int offsety = 0;
    // select the communication label color
    if(erase)
      gc.setColor(Settings.COLOR_BG);
    else 
      if(packet instanceof UDPPacket) {
        gc.setColor(Settings.COLOR_P_UDP);
        offsetx = 1; offsety = 1;
      }
      else if(packet instanceof TCPPacket) {
        gc.setColor(Settings.COLOR_P_TCP);
        offsetx = -1; offsety = -1;
      }
      else if(packet instanceof ICMPPacket) gc.setColor(Settings.COLOR_P_ICMP);
      else gc.setColor(Settings.COLOR_P_UNKNOWN);
    
    // draw a vector connecting the hosts communicating
    gc.drawLine(ax + offsetx, ay + offsety, bx + offsetx, by + offsety);

    // protocol label starts at the midpoint between two hosts communicating
    int midX = (ax + bx) / 2; int midY = (ay + by) / 2;

    // if communication is occurring on multiple ports, offset the mid-point
    while(slots.contains("" + midX + "" + midY)) {
      midX += offset;
      midY += offset; 
    }
    slots.add("" + midX + "" + midY);

    // select the label color
    if(erase)
      gc.setColor(Settings.COLOR_BG);
    else 
      if(packet instanceof UDPPacket) 
        gc.setColor(Settings.COLOR_P_UDP_H);
      else if(packet instanceof TCPPacket) 
        gc.setColor(Settings.COLOR_P_TCP_H);
      else if(packet instanceof ICMPPacket) 
        gc.setColor(Settings.COLOR_P_ICMP_H);
      else gc.setColor(Settings.COLOR_P_UNKNOWN_H);

    // label the communication vector
    gc.drawString(description + 
                  (Settings.SHOW_COUNTS ? 
                  " (" + connectionCount + ", " + byteCount + ")" : ""), 
                  midX, midY);
  }

  /**
   * Fetch a description (read protocol name) of the communication 
   * associated with this renderer.
   */
  public String getDescription() {
    return description;
  }

  public void incrementConnectionCount() {
    connectionCount ++;
  }

  public void incrementByteCount(long newBytes) {
    byteCount += newBytes;
  }

  /**
   * Generate a port 'key'.
   * <p>
   * Usually, the lower-numbered port is the service port while the 
   * higher-numbered port is the client's outgoing port.
   */
  static int keyPort(int srcPort, int dstPort) {
    return srcPort < dstPort ? srcPort : dstPort;
  }

  /**
   * Generate a communication 'key'.
   * <p>
   * Sorts the ip address and appends the port number to produce a 
   * hash key unique to the communication.
   */
  static String commKey(String srcAddr, String dstAddr, int port) {
    return (srcAddr.hashCode() < dstAddr.hashCode()) ? 
      (srcAddr + dstAddr + port) : (dstAddr + srcAddr + port);
  }


  Packet packet;
  HostRenderer hostA;
  HostRenderer hostB;
  String description;
  int connectionCount = 1;
  long byteCount = 0;
  protected static HashSet slots = new HashSet();
  private static int offset = 10;

  Graphics gc;
  private Canvas canvas;

  private String _rcsid = 
    "$Id: CommRenderer.java,v 1.3 2001/05/28 18:24:28 pcharles Exp $";
}

