// $Id: EthernetPacket.java,v 1.6 2001/05/28 18:30:12 pcharles Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.net;

import net.ultrametrics.console.AnsiEscapeSequences;


/**
 * An ethernet packet.
 * <p>
 * Contains link-level information in an ethernet packet. 
 * There are currently two subclasses. IP and ARP protocols are supported.
 * IPPacket extends with ip header and data information.
 * ARPPacket extends with hardware and protocol addresses.
 *
 * @author Patrick Charles and Jonas Lehmann
 * @version $Revision: 1.6 $
 * @lastModifiedBy $Author: pcharles $
 * @lastModifiedAt $Date: 2001/05/28 18:30:12 $
 */
public class EthernetPacket extends Packet
{
  /**
   * Construct a new, unrecognized ethernet packet.
   * <p>
   * If the ethernet packet type is not recognized, the 
   * ethernet type field is populated with 'unknown'.
   */
  public EthernetPacket(int lLen, byte [] bytes,
                        String macSourceAddress, 
                        String macDestinationAddress) {
    this(lLen, bytes, macSourceAddress, macDestinationAddress, 
         EthernetProtocols.ETHERNET_UNKNOWN);
  }

  /**
   * Construct a new, recognized ethernet packet.
   * <p>
   * When the type of the ethernet packet is known, it can be 
   * subclassified into a deriving class like IPPacket. The subclass then 
   * sets the ethernet type code to the appropriate and corresponding value.
   */
  public EthernetPacket(int lLen, byte [] bytes,
                        String macSourceAddress, 
                        String macDestinationAddress,
                        int typeCode) {
    this.header = PacketEncoding.extractHeader(0, lLen, bytes);
    this.data = PacketEncoding.extractData(0, lLen, bytes);
    this.macSourceAddress = macSourceAddress;
    this.macDestinationAddress = macDestinationAddress;
    this.typeCode = typeCode;
  }

  /**
   * Fetch the ethernet header, excluding data payload.
   */
  public byte [] getHeader() {
    return header;
  }

  /**
   * Fetch data portion of the ethernet header.
   */
  public byte [] getData() {
    return data;
  }

  public String getSourceHwAddress() {
    return macSourceAddress;
  }

  public String getDestinationHwAddress() {
    return macDestinationAddress;
  }

  /**
   * Fetch the code stored in the ethernet type field which indicates the 
   * type of datagram embedded in this ethernet packet.
   */
  public int getType() {
    return typeCode;
  }

  /**
   * Convert this ethernet packet to a readable string.
   */
  public String toString() {
    StringBuffer buffer = new StringBuffer();
    buffer.append('[');
    buffer.append(AnsiEscapeSequences.DARK_GRAY);
    buffer.append("EthernetPacket");
    buffer.append(AnsiEscapeSequences.RESET);
    buffer.append(": ");
    buffer.append(macSourceAddress + " -> " + 
                  macDestinationAddress);
    buffer.append(" l=" + header.length + "," + data.length);
    buffer.append(']');

    return buffer.toString();
  }

  /**
   * MAC source address.
   */
  String macSourceAddress;

  /**
   * MAC destination address.
   */
  String macDestinationAddress;

  /**
   * Ethernet type field.
   */
  int typeCode;

  /**
   * Ethernet header.
   */
  byte [] header;

  /**
   * Ethernet data.
   */
  byte [] data;

  private String _rcsid = 
    "$Id: EthernetPacket.java,v 1.6 2001/05/28 18:30:12 pcharles Exp $";
}
